# Generated by Django 4.2.1 on 2023-05-25 17:41

import datetime
from django.db import migrations, models
import django_tasks.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DocTask',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=80)),
                ('scheduled_at', models.DateTimeField(default=datetime.datetime.now)),
                ('completed_at', models.DateTimeField(null=True)),
                ('inputs', models.JSONField(default=dict, encoder=django_tasks.models.DefensiveJsonEncoder)),
                ('document', models.JSONField(default=list, encoder=django_tasks.models.DefensiveJsonEncoder)),
            ],
        ),
    ]
